#!/bin/python

from pathlib import Path
import shutil
import toml
import os
from setup import (
    app_name,
)
from anarchy_bot.config import (
    app_version,
)
from setup import (
    dependencies,
    app_name,
)


app_path = Path(
    __file__
).parent.resolve()
pypr_path = app_path / 'pyproject.toml'
dist_path = app_path / 'dist'

pypr_data = {
    'build-system': {
        'requires': ['hatchling'],
        'build-backend': 'hatchling.build',
    },
    'project': {
        'dependencies': dependencies,
        'name': app_name,
        'version': app_version,
        'authors': [
            {
                'name': 'gmanka',
                'email': 'gmankab@gmail.com',
            },
        ],
        'description': 'telegram bot that can add users to minecraft whitelist',
        'readme': 'readme.md',
        'requires-python': '>=3.10',
        'classifiers': [
            'Programming Language :: Python :: 3',
            'Operating System :: OS Independent',
        ],
        'urls': {
            'Documentation': 'https://t.me/gmanka',
            'Bug Tracker': 'https://t.me/gmanka',
            'Homepage': 'https://t.me/gmanka',
        }
    }
} 


with open(
    pypr_path,
    'w',
) as file:
    file.write(
        toml.dumps(
            pypr_data
        )
    )

shutil.rmtree(
    dist_path,
    ignore_errors = True,
)
os.system('python -m hatchling build')
for file in dist_path.iterdir():
    if file.suffix != '.whl':
        file.unlink()
os.system('python -m prettygit')




import datetime
import rich.console
import gmanka_yml as yml
import pyrogram as pg
import datetime
import pyrogram.enums
from pathlib import Path
from lang import t
from pyrogram import errors
from pyrogram.client import Client
from pyrogram.types import (
    InlineKeyboardButton as Ikb,
    InlineKeyboardMarkup as Ikm,
    InlineQuery,
    CallbackQuery,
    ChatPreview,
    ChatMember,
    Message,
    User,
    Chat,
)
from config import (
    app_version,
    chats_path,
    config,
    get,
    c,
)


c.log(f'imported [deep_sky_blue1]{__file__}')


class Chats:
    def __init__(
        self,
        path: Path,
    ) -> None:
        self.path = path
        data: dict = yml.from_file( # type: ignore
            self.path,
            default = {},
            expected_type = dict,
        )
        users_list: list[int] = get(
            data = data,
            item = 'users',
            default = [],
            expected_type = list,
        )
        self.chats_dict: dict[int, list[int]] = get(
            data = data,
            item = 'chats',
            default = {},
            expected_type = dict,
        )
        self.users_dict = {}
        for user in users_list:
            self.users_dict[user] = None

    async def remember_user(
        self,
        user_id: int,
    ) -> None:
        if user_id in self.users_dict:
            return
        self.users_dict[user_id] = None
        self.to_disk()
        await l.log(f'success: new user {user_id} writed to {self.path}')

    async def remember_chat(
        self,
        client: Client,
        chat: Chat,
    ) -> None:
        if chat.id in self.chats_dict:
            return
        ids: list[int] = []
        member: ChatMember | None = None
        async for member in client.get_chat_members( # type: ignore
            chat_id = chat.id,
            filter = pyrogram.enums.ChatMembersFilter.ADMINISTRATORS,
        ):
            if not member:
                continue
            if member.user.is_bot:
                continue
            ids.append(
                member.user.id
            )
        self.chats_dict[chat.id] = ids
        self.to_disk()
        await l.log(f'success: new chat {chat.id} writed to {self.path}')

    async def list_chat_admins(
        self,
        client: Client,
        chat: Chat,
    ) -> list[ChatMember]:
        chat_members_unsorted: list[ChatMember] = []
        chat_members_sorted: list[ChatMember] = []
        ids = []
        async for member in client.get_chat_members( # type: ignore
            chat_id = chat.id,
            filter = pyrogram.enums.ChatMembersFilter.ADMINISTRATORS,
        ):
            if not member:
                continue
            if member.user.is_bot:
                continue
            chat_members_unsorted.append(
                member
            )
        for member_id in self.chats_dict[chat.id]:
            for member in chat_members_unsorted:
                if member.user.id == member_id:
                    chat_members_sorted.append(member)
                    chat_members_unsorted.remove(member)
                    break
        chat_members_sorted += chat_members_unsorted
        for member in chat_members_sorted:
            ids.append(member.user.id)
        self.chats_dict[chat.id] = ids
        self.to_disk()
        return chat_members_sorted


    def to_disk(
        self,
    ) -> None:
        if not self.path:
            c.log(hl('error: not writed because path not specified'))
            return
        self.path.parent.mkdir(
            parents = True,
            exist_ok = True,
        )
        yml.to_file(
            data = {
                'version': app_version,
                'users': list(self.users_dict.keys()),
                'chats': self.chats_dict,
            },
            path = self.path,
        )


class Logs:
    def __init__(self) -> None:
        self.logs_chat: Chat | None = None

    async def init(
        self,
        client: Client,
        logs_chat: Chat | None = None,
    ) -> None:
        self.client: Client = client
        if logs_chat:
            config.logs_chat_id = logs_chat.id
            self.logs_chat: Chat | None = logs_chat
            config.to_disk()
            return
        if not config.logs_chat_id:
            return
        try:
            __logs_chat = await client.get_chat(
                config.logs_chat_id
            )
        except:
            await self.notify(
                f'failed get logs chat {config.logs_chat_id}'
            )
            return
        if isinstance(
            __logs_chat,
            ChatPreview,
        ):
            await self.notify(
                f'got chat preview instead of logs chat {config.logs_chat_id}'
            )
        elif isinstance(
            __logs_chat,
            Chat,
        ):
            self.logs_chat = __logs_chat
        else:
            raise TypeError(
                'unexpected logs chat object type'
            )

    async def remove(
        self,
    ) -> None:
        self.logs_chat = None
        config.logs_chat_id = 0
        config.to_disk()

    async def notify(
        self,
        text: str,
        chats_list: list[int] | None = None,
        reply_markup: Ikm | None = None,
    ) -> None:
        if not reply_markup:
            reply_markup = get_buttons([['main_menu_button']], None)
        if not self.client:
            raise ValueError(
                'no client'
            )
        c.log(hl(text))
        if not chats_list:
            chats_list = config.admins.copy()
        if self.logs_chat and self.logs_chat.id not in chats_list:
            chats_list.append(self.logs_chat.id)
        for chat_id in chats_list:
            try:
                await self.client.send_message(
                    chat_id = chat_id,
                    text = text,
                    reply_markup = reply_markup,
                )
            except:
                pass

    async def log(
        self,
        text: str | None = None,
        file: str | None = None,
        to_print: bool = True,
    ) -> str:
        if not text and not file:
            raise ValueError(
                'no text and no file'
            )
        if not text:
            text = ''
        elif to_print:
            c.log(hl(text))
        if file:
            c.log(f'file: {file}')
        if not self.logs_chat:
            c.log('no logs chat')
            return ''
        if not self.client:
            c.log(hl('error: no client'))
            return ''
        msg: Message | None = None
        if file:
            try:
                msg = await self.client.send_document(
                    chat_id = self.logs_chat.id,
                    document = file,
                    caption = text,
                )
            except errors.PeerIdInvalid:
                await l.remove()
                await l.log(f'error: can not get logs chat, removing it...')
            except Exception:
                c.log(f'''
error: failed send log with file to {self.logs_chat.id}
{write_error()}
'''
                )
        else:
            try:
                msg = await self.client.send_message(
                    chat_id = self.logs_chat.id,
                    text = text,
                )
            except errors.PeerIdInvalid:
                await l.remove()
                await l.log(f'error: can not get logs chat, removing it...')
            except Exception:
                c.log(f'''
failed send log to {self.logs_chat.id}
{write_error()}
'''
                )
        if msg:
            return url(msg.chat.id) + f'/{msg.id}'
        else:
            return ''


def get_buttons(
    buttons: list[list[str]],
    user: Message | CallbackQuery | InlineQuery | User | None,
) -> Ikm:
    parsed_buttons = []
    for row in buttons:
        parsed_row = []
        for button in row:
            if button[-2:] == '_0':
                clear_button = button[:-2]
            else:
                clear_button = button
            parsed_row.append(
                Ikb(
                    text = t(
                        item = clear_button,
                        user = user,
                    ),
                    callback_data = button,
                )
            )
        parsed_buttons.append(parsed_row)
    return Ikm(parsed_buttons)


def write_error() -> str:
    max_errors_in_dir = 30
    config.errors_path.mkdir(
        exist_ok = True,
        parents = True,
    )
    all_errors = list(
        config.errors_path.iterdir()
    )
    all_errors.sort()
    while len(
        all_errors
    ) >= max_errors_in_dir:
        all_errors[0].unlink()
        all_errors.remove(all_errors[0])
    file_date = datetime.datetime.now().strftime("%Y.%m.%d_%H.%M")
    test_error_path = config.errors_path / file_date
    error_path = Path(
        f'{test_error_path}.txt'
    )
    index = 2
    while error_path.exists():
        error_path = Path(
            f'{test_error_path}_{index}.txt'
        )
        index += 1
    with open(
        error_path,
        'w',
        encoding = "utf-8",
    ) as file:
        c_error = rich.console.Console(
            width = 80,
            file = file,
        )
        c_error.print_exception(
            show_locals = True,
            suppress = [pg],
        )
    return str(error_path)


def get_main_message(
    user: Message | CallbackQuery | User,
) -> dict:
    buttons = []
    reply_markup = None
    if isinstance(
        user,
        Message,
    ):
        u = user.from_user
    elif isinstance(
        user,
        CallbackQuery,
    ):
        u = user.from_user
    elif isinstance(
        user,
        User,
    ):
        u = user
    else:
        raise TypeError(
            f'{type(user)} is unexpected type for usr_msg'
        )
    if u.id in config.admins:
        buttons.append(['admin_panel_button'])
    if buttons:
        reply_markup = get_buttons(
            buttons,
            user,
        )
    else:
        reply_markup = None
    text = t('help_msg', user)
    text = text.strip('"""').strip('"""\n')
    return {
        'text': text,
        'reply_markup': reply_markup,
    }


async def filter_admin(
    item: CallbackQuery | Message,
):
        
    if item.from_user.id in config.admins:
        return False
    if isinstance(
        item,
        CallbackQuery,
    ):
        await item.answer(
            text = t('denied', item)
        )
    elif isinstance(
        item,
        Message,
    ):
        await item.reply(
            text = t('denied', item)
        )
    else:
        raise TypeError('wrong item type')
    return True


def url(
    chat_id: int | str,
) -> str:
    return 't.me/c/' + str(chat_id).replace(
        '-100',
        '',
    ).replace(
        '-',
        '',
    )


def mention(
    user: User,
) -> str:
    if user.username:
        return f'@{user.username}'
    else:
        return user.mention()


async def bad_button(
    cb: CallbackQuery,
) -> None:
    await cb.answer(
        text = t('bad_button_cb', cb),
    )


async def cb_bugreport(
    cb: CallbackQuery,
    bug: str,
) -> None:
    await cb.answer(
        bug,
        show_alert = True,
    )


async def msg_bugreport(
    msg: Message,
    bug: str,
) -> None:
    await msg.reply(
        bug,
        quote = True,
    )


async def get_end_int(
    data: CallbackQuery | Message,
) -> int | None:
    if isinstance(
        data,
        CallbackQuery,
    ):
        data_str = str(data.data)
        bugreport = cb_bugreport
    elif isinstance(
        data,
        Message,
    ):
        data_str = str(chats.users_dict[data.from_user.id])
        bugreport = msg_bugreport
    end = data_str.rsplit('_', 1)[-1]
    if end.replace('-', '').isdigit():
        return int(end)
    else:
        await bugreport(
            data, # type: ignore
            f'error: got non-int "{end}" in "{data_str}"',
        )
        return


async def get_two_end_int(
    cb: CallbackQuery,
) -> list[int] | None:
    items = str(cb.data).rsplit('_', 2)[-2:]
    result = []
    for i in items:
        if not i.replace('-', '').isdigit():
            await cb.answer(
                f'error: got non-int "{i}" in "{cb.data}"',
                show_alert = True,
            )
            return
        result.append(int(i))
    return result


def hl(
    text: str,
) -> str:
    return text.replace(
        'error:',
        '[red]error:[/red]',
    ).replace(
        'success:',
        '[green]success:[/green]',
    )


chats = Chats(
    chats_path
)
l = Logs()




import pyrogram as pg
from lang import t
from pyrogram.client import Client
from pyrogram.types import (
    InlineKeyboardButton as Ikb,
    InlineKeyboardMarkup as Ikm,
    CallbackQuery,
    Message,
)
from config import (
    config,
    c,
)
from common import (
    get_buttons,
    mention,
    url,
    l,
)


c.log(f'imported [deep_sky_blue1]{__file__}')


async def ask_to_set_logs_chat_msg(
    client: Client,
    msg: Message,
):
    if msg.from_user.id not in config.admins:
        await msg.reply(t('denied', msg))
        return
    if l.logs_chat and msg.chat.id == l.logs_chat.id:
        await msg.reply('already using this chat for logs')
        return
    done_button = t('done_button', msg)
    if msg.chat.type != pg.enums.ChatType.SUPERGROUP:
        await client.send_message(
            chat_id = msg.chat.id,
            text = f'''
to use this chat as logs chat, you must convert it to supergroup

1) open chat settings
2) enable topics
3) open chat settings again and disable topics
4) press {done_button}
''',
            reply_markup = Ikm([[Ikb(
                text = done_button,
                callback_data = f'confirm_set_logs_chat_button',
            )]])
        )
        return True
    await client.send_message(
        chat_id = msg.chat.id,
        text = t('ask_to_set_logs_chat_msg', msg).format(
            title = msg.chat.title,
        ),
        reply_markup = get_buttons(
            [['confirm_set_logs_chat_button']],
            msg,
        ),
    )
    return


async def setlogs_msg(
    client: Client,
    msg: Message,
):
    if msg.chat.type != pg.enums.ChatType.SUPERGROUP:
        await msg.reply(
            'wrong chat type, expected supergroup, got ' + str(msg.chat.type).lower()
        )
        return
    await l.init(
        client = client,
        logs_chat = msg.chat,
    )
    await l.notify(
        text = f'{mention(msg.from_user)} set new logs chat',
        reply_markup = Ikm([
            [Ikb(
                t('open_logs_chat_button', msg),
                url = url(msg.chat.id),
            )],
            [Ikb(
                t('main_menu_button', msg),
                'main_menu_button',
            )],
        ])
    )


async def confirm_set_logs_chat_button(
    client: Client,
    cb: CallbackQuery,
) -> None:
    if cb.message.chat.type != pg.enums.ChatType.SUPERGROUP:
        await cb.answer(
            'wrong chat type, expected supergroup, got ' + str(cb.message.chat.type).lower()
        )
        return
    await l.init(
        client = client,
        logs_chat = cb.message.chat,
    )
    await l.notify(
        text = f'{mention(cb.from_user)} set new logs chat',
        reply_markup = Ikm([
            [Ikb(
                t('open_logs_chat_button', cb),
                url = url(cb.message.chat.id),
            )],
            [Ikb(
                t('main_menu_button', cb),
                'main_menu_button',
            )],
        ])
    )


async def remove_logs_chat_button(
    client: Client,
    cb: CallbackQuery,
) -> None:
    await l.notify(f'{mention(cb.from_user)} removed logs chat')
    await l.remove()
    await logs_chat_button(
        client,
        cb,
    )


async def logs_chat_button(
    client: Client,
    cb: CallbackQuery,
) -> None:
    if l.logs_chat:
        await cb.message.edit(
            text = t('show_logs_chat_msg', cb),
            reply_markup = Ikm([
                [Ikb(
                    t('open_logs_chat_button', cb),
                    url = url(l.logs_chat.id),
                )],
                [Ikb(
                    t('remove_logs_chat_button', cb),
                    'remove_logs_chat_button',
                )],
                [Ikb(
                    t('replace_logs_chat_button', cb), 
                    url = f't.me/{cb.message.from_user.username}?startgroup=logs',
                )],
                [Ikb(t(
                    'back_to_admin_panel_button', cb),
                    'admin_panel_button',
                )],
            ])
        )
    else:
        await cb.message.edit(
            text = t('set_logs_chat_msg', cb),
            reply_markup = Ikm([
                [Ikb(
                    t('set_logs_chat_button', cb),
                    url = f't.me/{cb.message.from_user.username}?startgroup=logs',
                )],
                [Ikb(
                    t('back_to_admin_panel_button', cb),
                    'admin_panel_button',
                )]
            ])
        )


async def notify_removed(
    msg: Message,
) -> None:
    if msg.left_chat_member.id != config.me.id:
        if msg.chat.id == config.logs_chat_id:
            await l.remove()
            await l.notify(
                f'{mention(msg.from_user)} removed me from logs chat'
            )
    return




import pyrogram as pg
from lang import t
from pyrogram.client import Client
from pyrogram.types import (
    InlineKeyboardButton as Ikb,
    InlineKeyboardMarkup as Ikm,
    CallbackQuery,
    ChatPreview,
    Message,
    Chat,
    User,
)
from config import (
    config,
    c,
)
from common import (
    get_two_end_int,
    get_end_int,
    write_error,
    mention,
    url,
    chats,
)


c.log(f'imported [deep_sky_blue1]{__file__}')


async def pages_button(
    client: Client,
    cb: CallbackQuery,
    page_size: int = 5,
) -> None:
    chat_type = str(cb.data).replace(
        '_pages_button',
        '',
    ).replace(
        'list_bot_',
        '',
    )
    all_items = list(chats.users_dict.keys()) # TODO
    pages_count = (len(all_items) + page_size - 1) // page_size
    index = 0
    buttons = []
    while index + 1 <= pages_count:
        buttons.append([])
        for _ in range(0, 10):
            if index + 1 <= pages_count:
                buttons[-1].append(Ikb(
                    text = str(index + 1),
                    callback_data = f'list_bot_{chat_type}_button_{index}',
                ))
                index += 1
    await cb.edit_message_text(
        text = t('choose_page_msg', cb),
        reply_markup = Ikm(buttons),
    )


async def list_bot_users_button(
    client: Client,
    cb: CallbackQuery,
) -> None:
    opened_page = await get_end_int(cb)
    if opened_page is None:
        return
    users_dict: dict[int, str] = {}
    users_ids = list(chats.users_dict.keys())
    users_ids_chunk = await get_chunk(
        items_ids_list = users_ids,
        opened_page = opened_page,
        cb = cb,
    )
    if not users_ids:
        await cb.answer(
            t('no_bot_users_cb', cb),
            show_alert = True,
        )
        return
    if not users_ids_chunk:
        return
    for user_id in users_ids_chunk:
        try:
            user = await client.get_users(
                user_id
            )
        except Exception:
            users_dict[user_id] = str(user_id)
            continue
        if isinstance(
            user,
            list,
        ):
            user = user[0]
        if user.username:
            users_dict[user.id] = f'@{user.username}'
        else:
            users_dict[user.id] = user.first_name
    await get_list(
        page = opened_page,
        items_dict = users_dict,
        chat_type = 'users',
        all_items = users_ids,
        cb = cb,
    )


async def list_bot_admins_button(
    client: Client,
    cb: CallbackQuery,
) -> None:
    opened_page = await get_end_int(cb)
    if opened_page is None:
        return
    if not config.admins:
        await cb.answer(
            t('no_bot_admins_cb', cb),
            show_alert = True,
        )
        return
    admins_dict: dict[int, str] = {}
    admins_ids_chunk = await get_chunk(
        items_ids_list = config.admins,
        opened_page = opened_page,
        cb = cb,
    )
    if not admins_ids_chunk:
        return
    for admin_id in admins_ids_chunk:
        try:
            admin = await client.get_users(
                admin_id
            )
        except Exception:
            admins_dict[admin_id] = str(admin_id)
            continue
        if isinstance(
            admin,
            list,
        ):
            admin = admin[0]
        if admin.username:
            admins_dict[admin.id] = f'@{admin.username}'
        else:
            admins_dict[admin.id] = admin.first_name
    await get_list(
        page = opened_page,
        items_dict = admins_dict,
        chat_type = 'admins',
        all_items = config.admins,
        cb = cb,
        extra_button = Ikb(
            t('add_admin_button', cb),
            switch_inline_query = '',
        )
    )


async def list_bot_chats_button(
    client: Client,
    cb: CallbackQuery,
) -> None:
    opened_page = await get_end_int(cb)
    if opened_page is None:
        return
    if not chats.chats_dict:
        await cb.answer(
            t('no_bot_chats_cb', cb),
            show_alert = True,
        )
        return
    chats_dict: dict[int, str] = {}
    chats_ids_list = list(chats.chats_dict.keys())
    chats_ids_chunk = await get_chunk(
        items_ids_list = chats_ids_list,
        opened_page = opened_page,
        cb = cb,
    )
    if not chats_ids_chunk:
        return
    for chat_id in chats_ids_chunk:
        try:
            chat = await client.get_chat(chat_id)
        except:
            chat = None
            await cb.message.reply_document(
                document = write_error(),
                caption = f'failed get chat {chat_id}',
                quote = True,
            )
        if chat:
            if isinstance(
                chat,
                Chat,
            ):
                if chat.username:
                    chats_dict[chat_id] = f'@{chat.username}'
                else:
                    chats_dict[chat_id] = chat.title
            else:
                chats_dict[chat_id] = chat.title
        else:
            chats_dict[chat_id] = str(chat_id)
    await get_list(
        page = opened_page,
        items_dict = chats_dict,
        chat_type = 'chats',
        all_items = chats_ids_list,
        cb = cb,
        extra_button = Ikb(
            t('add_to_new_chat_button', cb),
            url = f't.me/{cb.message.from_user.username}?startgroup',
        )
    )


async def get_list(
    page: int,
    items_dict: dict[int, str],
    chat_type: str,
    all_items: list,
    cb,
    extra_button: Ikb | None = None,
) -> None:
    pages_count = (
        len(all_items) + config.page_size - 1
    ) // config.page_size
    buttons = []
    for item_id, item_name in items_dict.items():
        buttons.append([Ikb(
            str(item_name),
            f'get_bot_{chat_type}_button_{item_id}_{page}',
        )])
    controls = []
    if page == 0:
        controls.append(
            Ikb('❌', 'first_page_button')
        )
    else:
        controls.append(Ikb(
            '◄',
            f'list_bot_{chat_type}_button_{page - 1}',
        ))
    controls.append(Ikb(
        f'{page + 1}/{pages_count}',
        f'list_bot_{chat_type}_pages_button',
    ))
    if page + 1 >= pages_count:
        controls.append(
            Ikb('❌', 'last_page_button')
        )
    else:
        controls.append(Ikb(
            '►',
            f'list_bot_{chat_type}_button_{page + 1}',
        ))
    buttons.append(controls)
    if extra_button:
        buttons.append([extra_button])
    buttons.append([Ikb(
        t('back_to_admin_panel_button', cb),
        'admin_panel_button',
    )])
    await cb.message.edit(
        text = t(f'there_are_bot_{chat_type}_msg', cb),
        reply_markup = Ikm(buttons),
    )


async def get_bot_users_button(
    client: Client,
    cb: CallbackQuery,
    chat_type: str,
) -> None:
    end = await get_two_end_int(cb)
    if end is None:
        return
    user_id, page = end
    try:
        user = await client.get_users(
            user_id
        )
    except Exception:
        await cb.message.reply_document(
            document = write_error(),
            caption = 'got error while getting user',
            quote = True,
        )
        return
    if isinstance(
        user,
        list,
    ):
        return await cb.answer('error: got list instead of user')
    buttons = Ikm([
        [Ikb(
            t('more_info_button', cb),
            f'more_info_{chat_type}_button_{user.id}_{page}',
        )],
        [Ikb(
            t('back_to_admin_panel_button', cb),
            'admin_panel_button',
        )],
        [Ikb(
            t(f'back_to_bot_{chat_type}_list_button', cb),
            f'list_bot_{chat_type}_button_{page}',
        )],
    ])
    text = mention(user)
    await cb.edit_message_text(
        text = text,
        reply_markup = buttons,
    )


async def get_bot_chats_button(
    client: Client,
    cb: CallbackQuery,
) -> None:
    end = await get_two_end_int(cb)
    if end is None:
        return
    chat_id, page = end
    buttons = Ikm([
        [Ikb(
            t('more_info_button', cb),
            f'more_info_chats_button_{chat_id}_{page}',
        )],
        [Ikb(
            t('back_to_admin_panel_button', cb),
            'admin_panel_button',
        )],
        [Ikb(
            t(f'back_to_bot_chats_list_button', cb),
            f'list_bot_chats_button_{page}',
        )],
    ])
    await cb.edit_message_text(
        text = url(chat_id),
        reply_markup = buttons,
    )


async def more_info_users_button(
    client: Client,
    cb: CallbackQuery,
    chat_type: str,
) -> None:
    end = await get_two_end_int(cb)
    if end is None:
        return
    user_id, page = end
    try:
        user = await client.get_users(
            user_id
        )
    except Exception:
        await cb.message.reply_document(
            document = write_error(),
            caption = 'got error while getting user',
            quote = True,
        )
        return
    if isinstance(
        user,
        list,
    ):
        return await cb.answer('error: got list instead of user')
    buttons = Ikm([
        [Ikb(
            t('back_to_admin_panel_button', cb),
            'admin_panel_button',
        )],
        [Ikb(
            t(f'back_to_bot_{chat_type}_list_button', cb),
            f'list_bot_{chat_type}_button_{page}',
        )],
        [Ikb(
            t('shrink_button', cb),
            f'get_bot_{chat_type}_button_{user.id}_{page}',
        )],
    ])
    text = f'''
username: @{user.username}
mention: {user.mention()}
id: {user.id}
phone_number: {user.phone_number}
first_name: {user.first_name}
last_name: {user.last_name}
is_deleted: {user.is_deleted}
is_premium: {user.is_premium}
online_status: {user.status}
last_online_date: {user.last_online_date}
language_code: {user.language_code}
dc_id: {user.dc_id}
restrictions: {user.restrictions}
'''
    await cb.edit_message_text(
        text = text,
        reply_markup = buttons,
    )


async def more_info_chats_button(
    client: Client,
    cb: CallbackQuery,
) -> None:
    end = await get_two_end_int(cb)
    if end is None:
        return
    chat_id, page = end
    try:
        chat = await client.get_chat(
            chat_id
        )
    except Exception:
        await cb.message.reply_document(
            document = write_error(),
            caption = 'got error while getting chat',
            quote = True,
        )
        return
    if isinstance(
        chat,
        ChatPreview,
    ):
        text = f'''
got chat preview

link: {url(chat_id)}
title: {chat.title}
id: {chat_id}
members_count: {chat.members_count}
'''
    else:
        text = f'''
username: @{chat.username}
id: {chat.id}
title: {chat.title}
first_name: {chat.first_name}
last_name: {chat.last_name}
dc_id: {chat.dc_id}
restrictions: {chat.restrictions}
members_count: {chat.members_count}
bio: {chat.bio}
description: {chat.description}
'''
    buttons = Ikm([
        [Ikb(
            t('back_to_admin_panel_button', cb),
            'admin_panel_button',
        )],
        [Ikb(
            t(f'back_to_bot_chats_list_button', cb),
            f'list_bot_chats_button_{page}',
        )],
    ])
    await cb.edit_message_text(
        text = text,
        reply_markup = buttons,
    )


async def get_chunk(
    items_ids_list: list[int],
    opened_page: int,
    cb: CallbackQuery,
) -> None | list[int]:
    first_item = config.page_size * opened_page
    chunk = items_ids_list[first_item : first_item + config.page_size]
    if chunk:
        return chunk
    else:
        await cb.answer(
            t('wrong_page_cb', cb),
        )
        return




from lang import t
from pyrogram.client import Client
from pyrogram.types import (
    InlineKeyboardButton as Ikb,
    InlineKeyboardMarkup as Ikm,
    CallbackQuery,
)
from config import (
    config,
    c,
)
from common import (
    get_main_message,
    filter_admin,
    get_end_int,
    get_buttons,
    write_error,
    bad_button,
    mention,
    chats,
    l,
)
from lists import (
    get_bot_users_button,
    more_info_chats_button,
    more_info_users_button,
    list_bot_admins_button,
    list_bot_users_button,
    list_bot_chats_button,
)
from chats import (
    confirm_set_logs_chat_button,
    remove_logs_chat_button,
    logs_chat_button,
)


c.log(f'imported [deep_sky_blue1]{__file__}')


async def catched_on_cb(
    client: Client,
    cb: CallbackQuery,
):
    try:
        await on_cb(
            client,
            cb,
        )
    except Exception:
        await l.log(
            file = write_error()
        )


async def on_cb(
    client: Client,
    cb: CallbackQuery,
) -> None:
    c.log(f'got callback {cb.data} from user {cb.from_user.id}')
    users = {
        'request_admin_rights_button': request_admin_rights_button,
        'back_main_button': back_main_button,
        'main_menu_button': main_menu_button,
        'buy_pass_button': not_coded_yet_button,
    }
    users_in = {
        'promote_to_admin_button_': promote_to_admin_button,
    }
    answers = {
        'first_page_button': 'first_page_cb',
        'last_page_button': 'last_page_cb',
    }
    admins = {
        'confirm_set_logs_chat_button': confirm_set_logs_chat_button,
        'remove_logs_chat_button': remove_logs_chat_button,
        'admin_panel_button': admin_panel_button,
        'logs_chat_button': logs_chat_button,
    }
    admins_in = {
        'list_bot_users_button_': list_bot_users_button,
        'list_bot_chats_button_': list_bot_chats_button,
        'list_bot_admins_button_': list_bot_admins_button,
        'more_info_chats_button_': more_info_chats_button,
    }
    admins_get_user = {
        'get_bot_admins_button_': 'admins',
        'get_bot_users_button_': 'users',
    }
    admins_more_info_user = {
        'more_info_admins_button_': 'admins',
        'more_info_users_button_': 'users',
    }
    for excpected_cb, func in users.items():
        if excpected_cb == cb.data:
            await func(client, cb)
            return await cb.answer(text = '')
    for excpected_cb, func in users_in.items():
        if excpected_cb in str(cb.data):
            await func(client, cb)
            return await cb.answer(text = '')
    for excpected_cb, answer in answers.items():
        if excpected_cb == cb.data:
            return await cb.answer(
                text = t(answer, cb)
            )
    for excpected_cb, func in admins.items():
        if excpected_cb == cb.data:
            if await filter_admin(cb):
                return
            await func(client, cb)
            return await cb.answer(text = '')
    for excpected_cb, func in admins_in.items():
        if excpected_cb in str(cb.data):
            if await filter_admin(cb):
                return
            await func(client, cb)
            return await cb.answer(text = '')
    for excpected_cb, chat_type in admins_get_user.items():
        if excpected_cb in str(cb.data):
            if await filter_admin(cb):
                return
            await get_bot_users_button(client, cb, chat_type)
            return await cb.answer(text = '')
    for excpected_cb, chat_type in admins_more_info_user.items():
        if excpected_cb in str(cb.data):
            if await filter_admin(cb):
                return
            await more_info_users_button(client, cb, chat_type)
            return await cb.answer(text = '')
    return await bad_button(cb)


async def back_main_button(
    client: Client,
    cb: CallbackQuery,
) -> None:
    await cb.edit_message_text(
        **get_main_message(cb)
    )


async def admin_panel_button(
    client: Client,
    cb: CallbackQuery,
) -> None:
    buttons = [
        [
            'list_bot_admins_button_0',
            'list_bot_users_button_0',
            'list_bot_chats_button_0',
        ],
        [
            'logs_chat_button',
        ],
        ['back_main_button'],
    ]
    await cb.edit_message_text(
        text = t('admin_panel_msg', cb),
        reply_markup = get_buttons(
            buttons,
            cb,
        ),
    )


async def request_admin_rights_button(
    client: Client,
    cb: CallbackQuery,
) -> None:
    if cb.from_user.id in config.admins:
        await cb.answer(
            text = t('chat_partner_must_request_cb', cb),
            show_alert = True,
        )
        return
    await cb.edit_message_text(
        text = t(
            'promote_to_admin_message',
            cb
        ).format(
            user = mention(cb.from_user)
        ),
        reply_markup = Ikm([[Ikb(
            text = t('promote_to_admin_button', cb),
            callback_data = f'promote_to_admin_button_{cb.from_user.id}',
        )]])
    )


async def promote_to_admin_button(
    client: Client,
    cb: CallbackQuery,
) -> None:
    if cb.from_user.id not in config.admins:
        await cb.answer(
            text = t('chat_partner_must_promote_cb', cb),
            show_alert = True,
        )
        return
    promoted_user_id = await get_end_int(cb)
    if promoted_user_id is None:
        return
    try:
        promoted_user = await client.get_users(
            promoted_user_id
        )
    except Exception:
        text = f'failed get user {promoted_user_id}'
        await cb.answer(text)
        await client.send_document(
            chat_id = cb.from_user.id,
            document = write_error(),
            caption = text,
        )
        return
    if isinstance(
        promoted_user,
        list,
    ):
        await cb.answer('error: got list')
        return
    if promoted_user_id not in config.admins:
        config.admins.append(promoted_user.id)
    config.to_disk()
    text = f'{mention(cb.from_user)} successfully promoted {mention(promoted_user)} to admin'
    await cb.edit_message_text(text)
    await l.notify(text)


async def not_coded_yet_button(
    client: Client,
    cb: CallbackQuery,
) -> None:
    await cb.answer(
        t('not_coded_yet', cb)
    )


async def main_menu_button(
    client: Client,
    cb: CallbackQuery,
) -> None:
    chats.users_dict[cb.from_user.id] = None
    await cb.message.reply(
        **get_main_message(cb)
    )




try:
    import uvloop
    uvloop.install()
except Exception:
    pass

import sys
import signal
import asyncio
from cb import catched_on_cb
from msg import catched_on_message
from inline import catched_on_inline
from contextlib import suppress
from pyrogram.client import Client
from pyrogram.handlers.message_handler import MessageHandler
from pyrogram.handlers.inline_query_handler import InlineQueryHandler
from pyrogram.handlers.callback_query_handler import CallbackQueryHandler
from config import (
    interactive_input,
    start_message,
    app_name,
    config,
    c,
)
from common import (
    write_error,
    l,
)


c.log(f'imported [deep_sky_blue1]{__file__}')


async def idle():
    while True:
        await asyncio.sleep(1)


async def main():
    while True:
        if not config.session:
            if not config.token:
                config.token = interactive_input(
                    f'please input bot token: ',
                )
                config.to_disk()
            client =  Client(
                name = app_name,
                api_id = 1,   
                api_hash = 'b6b154c3707471f5339bd661645ed3d6',
                bot_token = config.token,
                in_memory = True,
            )
            try:
                await client.start()
                config.session = str(
                    await client.export_session_string()
                )
                await config.init(client = client)
            except:
                c.log(f'[red]error:[/red] bad token {write_error()}')
                config.token = None
                continue
            config.to_disk()
            break
        client = Client(
            name = app_name,
            session_string = config.session
        )
        try:
            await client.start()
            await config.init(client = client)
        except:
            c.log(f'[red]error:[/red] bad session {write_error()}')
            config.session = ''
            continue
        break
    await l.init(client = client)
    await l.log(start_message)
    for handler, to_call in {
        MessageHandler: catched_on_message,
        CallbackQueryHandler: catched_on_cb,
        InlineQueryHandler: catched_on_inline,
    }.items():
        client.add_handler(
            handler(
                to_call
            )
        )
    await idle()


if __name__ == 'main':
    try:
        loop = asyncio.get_event_loop()
        loop.add_signal_handler(
            signal.SIGINT,
            loop.stop,
        )
        loop.run_until_complete(
            main()
        )
        for task in asyncio.all_tasks(loop):
            task.cancel()
        with suppress(
            asyncio.CancelledError
        ):
            loop.run_until_complete(loop.shutdown_asyncgens())
        loop.close()
    except (
        KeyboardInterrupt,
        RuntimeError,
    ):
        print(' exiting')
        sys.exit()




#!/bin/python3

from pathlib import Path
import urllib.request
import subprocess
import shutil
import venv
import sys


setup_py = Path(
    __file__
)
app_name = 'anarchy_bot'
app_path = Path(
    __file__
).parent.parent.resolve()
venv_path = app_path / '.venv'
activate_sh = venv_path / 'bin' / 'activate'
venv_python = venv_path / 'bin' / 'python'
start_sh = app_path / 'start.sh'
ssh_ip = '141.145.194.71'
ssh_username = 'bogdan'
pip = [
    venv_python,
    '-m',
    'pip',
]
dependencies = [
    'gmanka_yml==23.0.*',
    'pyrogram==2.0.*',
    'tgcrypto==1.2.*',
    'uvloop==0.17.*',
    'rich==13.4.*',
]


def green(
    text: str,
):
    return '\033[1;32m' + text + '\033[0m'

def red(
    text: str,
):
    return '\033[1;31m' + text + '\033[0m'


def install_libs():
    if venv_path.exists():
        shutil.rmtree(venv_path)

    venv.create(venv_path)
    get_pip_url = 'https://bootstrap.pypa.io/get-pip.py'
    response = urllib.request.urlopen(
        get_pip_url
    )
    total_size = int(
        response.headers.get('content-length', 0)
    )
    block_size = 1024 * 8
    count = 0
    get_pip = b''
    while True:
        chunk = response.read(block_size)
        if not chunk:
            break
        get_pip += chunk
        count += 1
        done = count * block_size
        percent_done = done * 100 // total_size
        print(
            f'\rdownloading get-pip.py: {percent_done}%',
            end = '',
        )
        sys.stdout.flush()
    print()
    python_process = subprocess.Popen(
        [venv_python],
        stdin = subprocess.PIPE,
    )
    python_process.communicate(
        input = get_pip
    )
    subprocess.run(
        [*pip, 'install', '-U', *dependencies]
    )


def main():
    install_libs()

    start_sh.write_text(
f'''\
#!/bin/bash
{venv_python} {app_name}
\n'''
    )

    start_sh.chmod(0o755)
    setup_py.chmod(0o755)


    print(green(
        'successfuly installed, use ./start.sh to start'
    ))


if __name__ == '__main__':
    main()




import pyrogram as pg
import pyrogram.enums
import pyrogram.errors
from lang import t
from pyrogram.client import Client
from pyrogram.types import (
    ChatPrivileges,
    ChatMember,
    Message,
)
from config import(
    yes_no,
    config,
    c,
)
from common import (
    get_main_message,
    filter_admin,
    write_error,
    mention,
    chats,
    l,
)
from chats import (
    ask_to_set_logs_chat_msg,
    notify_removed,
    setlogs_msg,
)


c.log(f'imported [deep_sky_blue1]{__file__}')


async def catched_on_message(
    client: Client,
    msg: Message,
):
    try:
        await on_message(
            client,
            msg,
        )
    except Exception:
        error_path = write_error()
        log_msg = await l.log(
            file = error_path
        )
        if not log_msg:
            log_msg = error_path
        await msg.reply(
            f'unknown error, please forward this message to @gmankachat\n\nlog: {log_msg}'
        )


def is_command(
    msg_text: str,
    command: str,
) -> bool:
    if msg_text.startswith(command + ''):
        return True
    if msg_text.startswith(f'{command}@{config.me.username}'):
        return True
    return False


async def on_message(
    client: Client,
    msg: Message,
) -> None:
    users = {
        '/h': help_msg,
        '/help': help_msg,
        '/becomeadmin': becomeadmin,
        '/ba': becomeadmin,
    }
    admins = {
        '/setlogs': setlogs_msg,
    }
    if msg.service:
        if msg.service == pg.enums.MessageServiceType.LEFT_CHAT_MEMBERS:
            await notify_removed(msg)
        return
    if not msg.text:
        return
    if not config.owner:
        await set_owner(
            msg,
        )
    if msg.chat.type == pg.enums.ChatType.PRIVATE:
        await chats.remember_user(msg.from_user.id)
        action = chats.users_dict[msg.from_user.id]
        if action is None:
            pass
    else:
        await chats.remember_chat(
            client,
            msg.chat,
        )
        start = {
            'logs': ask_to_set_logs_chat_msg,
        }
        for action, to_run in start.items():
            if msg.text.endswith('?startgroup=' + action):
                await to_run(client, msg)
                return
    for excpected_msg, to_run in users.items():
        if is_command(
            msg.text,
            excpected_msg,
        ):
            return await to_run(client, msg)
    for excpected_msg, to_run in admins.items():
        if is_command(
            msg.text,
            excpected_msg,
        ):
            if await filter_admin(msg):
                return
            return await to_run(client, msg)
    if msg.chat.type == pg.enums.ChatType.PRIVATE:
        await msg.reply(
            **get_main_message(msg)
        )
        return


async def help_msg(
    client: Client,
    msg: Message,
):
    await msg.reply(
        **get_main_message(msg),
        disable_web_page_preview = True,
    )
    if msg.from_user.id in config.admins:
        await msg.reply(
            t('owner_commands_msg', msg)
        )


async def becomeadmin(
    client: Client,
    msg: Message,
):
    splitted = msg.text.split(' ', 1)
    if len(splitted) == 1:
        await msg.reply(
            'you must specify your title after /becomeadmin\n\nexample: /becomeadmin bebra'
        )
        return
    title = splitted[-1][:16]
    if msg.chat.type != pyrogram.enums.ChatType.SUPERGROUP:
        await msg.reply(
            'wrong chat type, expected supergroup, got ' + str(msg.chat.type).lower()
        )
        return
    if not msg.from_user:
        await msg.reply(
            'you must send message as user, not as channel or chat'
        )
    text = f'trying to make {mention(msg.from_user)} an admin...'
    responce: Message = await msg.reply(
        text
    )
    while True:
        try:
            await client.promote_chat_member(
                chat_id = msg.chat.id,
                user_id = msg.from_user.id,
                privileges = ChatPrivileges(
                    can_manage_chat = True,
                    can_change_info = True,
                ),
            )
            text += f'\n\nsuccesfully promoted {mention(msg.from_user)} to admin'
            await responce.edit_text(text)
            break
        except pyrogram.errors.ChatAdminRequired:
            text += '\n\ni have no rights to make you an admin'
            await responce.edit_text(text)
            return
        except pyrogram.errors.UserCreator:
            text += '\n\nbro you are chat owner'
            await responce.edit_text(text)
            return
        except pyrogram.errors.AdminsTooMuch:
            admins: list[ChatMember] = await chats.list_chat_admins(
                client = client,
                chat = msg.chat,
            )
            demoted = False
            for admin in admins:
                try:
                    await client.promote_chat_member(
                        chat_id = msg.chat.id,
                        user_id = admin.user.id,
                        privileges = ChatPrivileges(
                            can_manage_video_chats = False,
                            can_restrict_members = False,
                            can_promote_members = False,
                            can_delete_messages = False,
                            can_post_messages = False,
                            can_edit_messages = False,
                            can_invite_users = False,
                            can_pin_messages = False,
                            can_change_info = False,
                            can_manage_chat = False,
                            is_anonymous = False,
                        ),
                    )
                    demoted = True
                    text += f'\n\ntoo many admins, demoting {mention(admin.user)}'
                    await responce.edit_text(text)
                    break
                except pyrogram.errors.UserCreator:
                    continue
                except Exception as e:
                    text += f'\n\nfailed to demote {mention(admin.user)}\n{e}'
                    await responce.edit_text(text)
                    continue
            if demoted:
                break
            text += f'\n\ntoo many admins, no admin available to demote'
            await responce.edit_text(text)
            return
    await client.set_administrator_title(
        chat_id = msg.chat.id,
        user_id = msg.from_user.id,
        title = title,
    )
    text += f'\n\nsuccesfully set title {title}'
    await responce.edit_text(text)


async def set_owner(
    msg: Message,
) -> None:
    await msg.reply(
        t('setup_msg', msg).format(
            username = config.me.username
        )
    )
    user = msg.from_user
    if yes_no(
        f'''
you have not added an admin yet

this user contacted bot:

username - @{user.username}
name - {user.first_name}
id - {user.id}

do you want to make him admin? [Y/n] \
'''
    ):
        config.owner = user.id
        if config.owner not in config.admins:
            config.admins.append(user.id)
        config.to_disk()
        await l.notify(f'success: new admin {user.id}')
    else:
        await l.log(f'canceled adding {user.id} to admins')
        await msg.reply(f'canceled adding {user.id} to admins')




import pyrogram as pg
from lang import t
from pyrogram.client import Client
from pyrogram.types import (
    InlineKeyboardButton as Ikb,
    InlineKeyboardMarkup as Ikm,
    InlineQueryResultArticle,
    InputTextMessageContent,
    CallbackQuery,
    InlineQuery,
    ChatPreview,
    Message,
    Chat,
    User,
)
from config import (
    config,
)
from common import (
    write_error,
    get_buttons,
    c,
    l,
)


c.log(f'imported [deep_sky_blue1]{__file__}')


async def catched_on_inline(
    client: Client,
    inline: InlineQuery,
):
    try:
        await on_inline(
            client,
            inline,
        )
    except Exception:
        await l.log(
            file = write_error()
        )


async def on_inline(
    client: Client,
    inline: InlineQuery,
) -> None:
    c.log(inline)
    if inline.from_user.id not in config.admins:
        await inline.answer(
            results = [],
            is_personal = True,
            cache_time = 0,
        )
    await inline.answer(
        results = [InlineQueryResultArticle(
            title = t('promote_to_admin_button', inline),
            input_message_content = InputTextMessageContent(
                t('request_admin_rights_message', inline),
            ),
            reply_markup = get_buttons([['request_admin_rights_button']], inline),
        )],
        is_personal = True,
        cache_time = 0,
    )




from pathlib import Path
from pyrogram.types import (
    CallbackQuery,
    InlineQuery,
    Message,
    User,
)
from config import (
    src_path,
    yml,
    c,
)


c.log(f'imported [deep_sky_blue1]{__file__}')


class TranslationError(Exception):
    pass


class Translation:
    def __init__(
            self,
            path: str | Path = f'{src_path}/lang',
            warns: bool = True,
        ) -> None:
        self.data = {}
        self.path = Path(path)
        self.warns = warns
        for file in Path(path).iterdir():
            self.data[file.stem] = yml.from_file(
                file
            )

    def translate(
        self,
        item: str,
        user: User | Message | InlineQuery | CallbackQuery | None,
    ) -> str:
        lang = 'en'
        if isinstance(
            user,
            User,
        ):
            lang = user.language_code
        for i in (
            Message,
            InlineQuery,
            CallbackQuery,
        ):
            if isinstance(
                user,
                i,
            ):
                lang = user.from_user.language_code
        if not lang:
            lang = 'en'
        if 'en' not in self.data:
            raise TranslationError(
                f'en not in {self.path}'
            )
        if item not in self.data['en']:
            raise TranslationError(
                f'{item} not in en'
            )
        if self.warns:
            if lang not in self.data:
                c.log(f'[yellow]warn:[/yellow] {lang} not in [deep_sky_blue1]{self.path}')
            elif item not in self.data[lang]:
                c.log(f'[yellow]warn:[/yellow] not in {lang}')
            lang = 'en'
        return self.data[lang][item]


translation = Translation()
t = translation.translate




from pyrogram.client import Client
from pathlib import Path
from typing import Any
import pyrogram as pg
import gmanka_yml as yml
import rich.console
import platform
from pyrogram.types import(
    User,
)


try:
    import uvloop
    uvloop_version = uvloop._version.__version__
except Exception:
    uvloop_version = None
try:
    import tgcrypto # type: ignore
    using_tgcrypto = True
except Exception:
    using_tgcrypto = False


app_name = 'anarchy_bot'
app_version = '23.0.5'
c = rich.console.Console()
c.log(f'imported [deep_sky_blue1]{__file__}')
app_path = Path(
    __file__
).parent.parent.resolve()
src_path = app_path / app_name
data_path = app_path / f'{app_name}_data'
wl_path = data_path / 'whitelist.yml'
chats_path = data_path / 'chats.yml'

os_name = platform.system()
if os_name == 'Linux':
    try:
        os_name = platform.freedesktop_os_release()['PRETTY_NAME']
    except Exception:
        pass


start_message = f'''\
gmanka {app_name} {app_version}
pyrogram {pg.__version__}
uvloop {uvloop_version}
tgcrypto {using_tgcrypto}
{platform.python_implementation()} {platform.python_version()}
{os_name} {platform.release()}
data path: `{data_path}`\
'''


def get(
    data: dict,
    item: str,
    default: Any,
    expected_type: Any,
) -> Any:
    got = data.get(
        item,
        default,
    )
    if isinstance(
        got,
        expected_type,
    ):
        return got
    else:
        return default
        

def yes_no(
    to_print: str
) -> bool:
    c.print(
        to_print,
        end = '',
    )
    yes_no = input()
    if yes_no in [
        '',
        'Y',
        'y',
    ]:
        return True
    else:
        return False


def interactive_input(
    to_print: str
) -> str:
    while True:
        c.print(
            to_print,
            end = '',
        )
        data = input()
        if data:
            if yes_no(
                f'[deep_sky_blue1]{data}[/deep_sky_blue1] - is it correct? [Y/n] ',
            ):
                return data


class Config:
    def __init__(
        self,
        data_path: Path,
    ) -> None:
        self.data_path:Path = data_path
        self.config_path: Path = data_path / f'{app_name}.yml'
        self.errors_path: Path = self.data_path / 'errors'
        self.page_size = 5
        data: dict = yml.from_file( # type: ignore
            path = self.config_path,
            default = {},
            expected_type = dict,
        )
        self.owner: int = get(
            data = data,
            item = 'owner',
            default = 0,
            expected_type = int,
        )
        self.logs_chat_id: int = get(
            data = data,
            item = 'logs_chat_id',
            default = 0,
            expected_type = int,
        )
        self.admins: list[int] = get(
            data = data,
            item = 'admins',
            default = [],
            expected_type = list,
        )
        self.session: str = get(
            data = data,
            item = 'session',
            default = '',
            expected_type = str,
        )
        self.token: str = get(
            data = data,
            item = 'token',
            default = '',
            expected_type = str | None,
        )
        if self.owner and self.owner not in self.admins:
            self.admins.append(self.owner)
        self.__me: User | None = None
    def __get_me(self) -> User:
        if self.__me == None:
            raise ValueError('me was not set')
        return self.__me

    async def init(
        self,
        client: Client,
    ) -> None:
        if not self.__me:
            self.__me = await client.get_me()

    def to_disk(
        self,
    ) -> None:
        if not self.config_path:
            c.log('[red] not writed because path not specified')
            return
        self.config_path.parent.mkdir(
            parents = True,
            exist_ok = True,
        )
        yml.to_file(
            data = {
                'version': app_version,
                'logs_chat_id': self.logs_chat_id,
                'admins': self.admins,
                'owner': self.owner,
                'token': self.token,
                'session': self.session,
            },
            path = self.config_path,
        )
        c.log(f'success: writed changes to [deep_sky_blue1]{self.config_path}')

    me: User = property(fget = __get_me) # type: ignore


config = Config(
    data_path = data_path,
)

